@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -17,22 +17,38 @@
 
 """Tests for the subscription service."""
 
+import sys
 import unittest
 
 from contextlib import suppress
+from datetime import datetime
+from lazr.config import as_timedelta
 from mailman.app.lifecycle import create_list
+from mailman.app.membership import delete_member
 from mailman.app.subscriptions import SubscriptionWorkflow
+from mailman.config import config
 from mailman.interfaces.address import InvalidEmailAddressError
 from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.mailinglist import SubscriptionPolicy
-from mailman.interfaces.member import MemberRole, MembershipIsBannedError
+from mailman.interfaces.member import (
+    AlreadySubscribedError,
+    DeliveryMode,
+    DeliveryStatus,
+    MemberRole,
+    MembershipIsBannedError,
+)
 from mailman.interfaces.pending import IPendings
-from mailman.interfaces.subscriptions import TokenOwner
+from mailman.interfaces.subscriptions import ISubscriptionManager, TokenOwner
 from mailman.interfaces.usermanager import IUserManager
 from mailman.testing.helpers import (
-    LogFileMark, get_queue_messages, set_preferred)
+    configuration,
+    get_queue_messages,
+    LogFileMark,
+    set_preferred,
+)
 from mailman.testing.layers import ConfigLayer
 from mailman.utilities.datetime import now
+from sqlalchemy import text
 from unittest.mock import patch
 from zope.component import getUtility
 
@@ -78,6 +94,55 @@ class TestSubscriptionWorkflow(unittest.TestCase):
         # The token is still in the database.
         self._expected_pendings_count = 1
 
+    @unittest.skipIf(sys.hexversion < 0x30700a0, 'No datetime.fromisoformat')
+    def test_pended_expiration_user(self):
+        # As test_pended_data, but here we're interested in the expiration
+        # which we need to get at a low level.
+        anne = self._user_manager.create_address(self._anne)
+        workflow = SubscriptionWorkflow(self._mlist, anne)
+        with suppress(StopIteration):
+            workflow.run_thru('send_confirmation')
+        self.assertIsNotNone(workflow.token)
+        pendable = getUtility(IPendings).confirm(workflow.token, expunge=False)
+        self.assertEqual(pendable['when'], '2005-08-01T07:49:23')
+        self.assertEqual(pendable['token_owner'], 'subscriber')
+        # Get the expiration datetime and verify.
+        expiration = config.db.store.execute(text(
+            "select expiration_date from pended where token = '{}';".format(
+                workflow.token))).fetchone()[0]
+        if isinstance(expiration, str):
+            expiration = datetime.fromisoformat(expiration)
+        expected = datetime.fromisoformat(pendable['when']) + as_timedelta(
+            config.mailman.pending_request_life)
+        self.assertEqual(expiration, expected)
+        # The token is still in the database.
+        self._expected_pendings_count = 1
+
+    @unittest.skipIf(sys.hexversion < 0x30700a0, 'No datetime.fromisoformat')
+    def test_pended_expiration_moderator(self):
+        # Test that a Pendable for the moderator has the right expiration.
+        self._mlist.subscription_policy = SubscriptionPolicy.moderate
+        anne = self._user_manager.create_address(self._anne)
+        anne.verified_on = datetime.now()
+        workflow = SubscriptionWorkflow(self._mlist, anne)
+        with suppress(StopIteration):
+            workflow.run_thru('get_moderator_approval')
+        self.assertIsNotNone(workflow.token)
+        pendable = getUtility(IPendings).confirm(workflow.token, expunge=False)
+        self.assertEqual(pendable['when'], '2005-08-01T07:49:23')
+        self.assertEqual(pendable['token_owner'], 'moderator')
+        # Get the expiration datetime and verify.
+        expiration = config.db.store.execute(text(
+            "SELECT expiration_date FROM pended WHERE token = '{}';".format(
+                workflow.token))).fetchone()[0]
+        if isinstance(expiration, str):
+            expiration = datetime.fromisoformat(expiration)
+        expected = datetime.fromisoformat(pendable['when']) + as_timedelta(
+            config.mailman.moderator_request_life)
+        self.assertEqual(expiration, expected)
+        # The token is still in the database.
+        self._expected_pendings_count = 1
+
     def test_user_or_address_required(self):
         # The `subscriber` attribute must be a user or address.
         workflow = SubscriptionWorkflow(self._mlist)
@@ -143,6 +208,42 @@ class TestSubscriptionWorkflow(unittest.TestCase):
         workflow = SubscriptionWorkflow(self._mlist, user)
         self.assertRaises(AssertionError, workflow.run_thru, 'sanity_checks')
 
+    def test_sanity_checks_finds_address_for_user(self):
+        # Test raises AlreadySubscribedError for User when member is Address.
+        anne = self._user_manager.make_user(self._anne)
+        anne.addresses[0].verified_on = anne.created_on
+        anne.preferred_address = anne.addresses[0]
+        self._mlist.subscription_policy = SubscriptionPolicy.open
+        # Subscribe Address.
+        ISubscriptionManager(self._mlist).register(
+            anne.preferred_address,
+            pre_verified=True,
+            pre_confirmed=True,
+            pre_approved=True,
+            send_welcome_message=False
+            )
+        workflow = SubscriptionWorkflow(self._mlist, anne)
+        self.assertRaises(
+            AlreadySubscribedError, workflow.run_thru, 'sanity_checks')
+
+    def test_sanity_checks_finds_user_for_address(self):
+        # Test raises AlreadySubscribedError for Address when member is User.
+        anne = self._user_manager.make_user(self._anne)
+        anne.addresses[0].verified_on = anne.created_on
+        anne.preferred_address = anne.addresses[0]
+        self._mlist.subscription_policy = SubscriptionPolicy.open
+        # Subscribe User.
+        ISubscriptionManager(self._mlist).register(
+            anne,
+            pre_verified=True,
+            pre_confirmed=True,
+            pre_approved=True,
+            send_welcome_message=False
+            )
+        workflow = SubscriptionWorkflow(self._mlist, anne.preferred_address)
+        self.assertRaises(
+            AlreadySubscribedError, workflow.run_thru, 'sanity_checks')
+
     def test_sanity_checks_globally_banned_address(self):
         # An exception is raised if the address is globally banned.
         anne = self._user_manager.create_address(self._anne)
@@ -327,7 +428,7 @@ class TestSubscriptionWorkflow(unittest.TestCase):
         self.assertEqual(workflow.token_owner, TokenOwner.no_one)
 
     def test_do_subscription_pre_approved(self):
-        # An moderation-requiring subscription policy plus a pre-verified and
+        # A moderation-requiring subscription policy plus a pre-verified and
         # pre-approved address means the user gets subscribed to the mailing
         # list without any further confirmations or approvals.
         self._mlist.subscription_policy = SubscriptionPolicy.moderate
@@ -346,7 +447,7 @@ class TestSubscriptionWorkflow(unittest.TestCase):
         self.assertEqual(workflow.token_owner, TokenOwner.no_one)
 
     def test_do_subscription_pre_approved_pre_confirmed(self):
-        # An moderation-requiring subscription policy plus a pre-verified and
+        # A moderation-requiring subscription policy plus a pre-verified and
         # pre-approved address means the user gets subscribed to the mailing
         # list without any further confirmations or approvals.
         self._mlist.subscription_policy = (
@@ -535,6 +636,21 @@ approval:
         # one token still in the database.
         self._expected_pendings_count = 1
 
+    def test_invitation_verp_confirmations_no(self):
+        # Test From: and Subject: with verp_confirmations equal no.
+        anne = self._user_manager.create_address(self._anne)
+        self.assertIsNone(anne.verified_on)
+        # Run the workflow to model the confirmation step.
+        workflow = SubscriptionWorkflow(self._mlist, anne, invitation=True)
+        with configuration('mta', verp_confirmations='no'):
+            list(workflow)
+        items = get_queue_messages('virgin', expected_count=1)
+        message = items[0].msg
+        token = workflow.token
+        self.assertEqual(f'confirm {token}', str(message['Subject']))
+        self.assertEqual('test-request@example.com', message['From'])
+        self._expected_pendings_count = 1
+
     def test_send_confirmation_pre_confirmed(self):
         # A confirmation message gets sent when the address is not verified
         # but the subscription is pre-confirmed.
@@ -555,6 +671,21 @@ approval:
         # one token still in the database.
         self._expected_pendings_count = 1
 
+    def test_confirmation_subscribe_verp_confirmations_no(self):
+        # Test From: and Subject: with verp_confirmations equal no.
+        anne = self._user_manager.create_address(self._anne)
+        self.assertIsNone(anne.verified_on)
+        # Run the workflow to model the confirmation step.
+        workflow = SubscriptionWorkflow(self._mlist, anne, pre_confirmed=True)
+        with configuration('mta', verp_confirmations='no'):
+            list(workflow)
+        items = get_queue_messages('virgin', expected_count=1)
+        message = items[0].msg
+        token = workflow.token
+        self.assertEqual(f'confirm {token}', str(message['Subject']))
+        self.assertEqual('test-request@example.com', message['From'])
+        self._expected_pendings_count = 1
+
     def test_send_confirmation_pre_verified(self):
         # A confirmation message gets sent even when the address is verified
         # when the subscription must be confirmed.
@@ -779,3 +910,79 @@ approval:
         self.assertEqual(approved_workflow.user, bill)
         # Run the workflow through.
         list(approved_workflow)
+
+    def test_set_member_prefs_and_subscribe(self):
+        # Test that we can set member's delivery_mode and delivery_status
+        # after subscribing them.
+        anne = self._user_manager.create_user(self._anne)
+        anne_address = anne.addresses[0]
+        workflow = SubscriptionWorkflow(
+            self._mlist, anne_address,
+            pre_verified=True, delivery_mode=DeliveryMode.plaintext_digests,
+            delivery_status=DeliveryStatus.by_user)
+        list(workflow)
+        # Anne's subcription is pending confirmation.
+        confirm_workflow = SubscriptionWorkflow(self._mlist)
+        confirm_workflow.token = workflow.token
+        confirm_workflow.restore()
+        list(confirm_workflow)
+        # Anne should be subscribed with the desired membership preferences.
+        anne_member = self._mlist.members.get_member(anne_address.email)
+        self.assertEqual(
+            anne_member.delivery_mode, DeliveryMode.plaintext_digests)
+        self.assertEqual(anne_member.delivery_status, DeliveryStatus.by_user)
+
+    def test_translated_subject_your_confirmation_is_needed(self):
+        anne = self._user_manager.create_user(self._anne)
+        old_language = self._mlist.preferred_language.code
+        self._mlist.preferred_language.code = 'fr'
+        anne_address = anne.addresses[0]
+        workflow = SubscriptionWorkflow(
+            self._mlist, anne_address, pre_verified=True,
+            send_welcome_message=False, delivery_status=DeliveryStatus.by_user)
+        list(workflow)
+        confirm_workflow = SubscriptionWorkflow(self._mlist)
+        confirm_workflow.token = workflow.token
+        confirm_workflow.restore()
+        list(confirm_workflow)
+        self._mlist.preferred_language.code = old_language
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertEqual(items[0].msg['Subject'], "Votre "
+                         "confirmation est nécessaire pour vous abonner à "
+                         "la liste de diffusion test@example.com.")
+
+    def test_translated_subject_unsubscribe(self):
+        anne = self._user_manager.create_user(self._anne)
+        set_preferred(anne)
+        old_code = anne.preferences.preferred_language
+        anne.preferences.preferred_language = 'fr'
+        self._mlist.subscribe(anne, send_welcome_message=False)
+        delete_member(self._mlist, self._anne, False, True)
+        anne.preferences.preferred_language = old_code
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertEqual(str(items[0].msg['Subject']), "Vous avez"
+                         " été désabonné de la liste de diffusion Test")
+
+    def test_mutltiple_subscriptions_for_one_user(self):
+        anne = self._user_manager.create_user(self._anne)
+        set_preferred(anne)
+        email_2 = self._user_manager.create_address('second@example.com')
+        anne.link(email_2)
+        # Subscribe the user first.
+        workflow = SubscriptionWorkflow(
+            self._mlist, anne,
+            pre_verified=True, pre_confirmed=True, pre_approved=True
+            )
+        list(workflow)
+        # Assert the user was subscribed.
+        member = self._mlist.regular_members.get_member(self._anne)
+        self.assertIsNotNone(member)
+        # Now, try to subscribe the 2nd address of the same user.
+        workflow = SubscriptionWorkflow(
+            self._mlist, email_2,
+            pre_verified=True, pre_confirmed=True, pre_approved=True
+            )
+        list(workflow)
+        # And assert it succeeded.
+        member = self._mlist.regular_members.get_member(email_2.email)
+        self.assertIsNotNone(member)
