@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2009-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -18,7 +18,6 @@
 """Fake MTA for testing purposes."""
 from __future__ import generator_stop
 
-import asyncio
 import smtplib
 
 from aiosmtpd.controller import Controller
@@ -41,7 +40,7 @@ class FakeMTA:
     def delete(self, mlist):
         pass
 
-    def regenerate(self, output=None):
+    def regenerate(self, directory=None):
         pass
 
 
@@ -54,14 +53,12 @@ class ConnectionCountingHandler(MessageHandler):
     def handle_message(self, message):
         self._msg_queue.put(message)
 
-    @asyncio.coroutine
-    def handle_EHLO(self, server, session, envelope, hostname):
+    async def handle_EHLO(self, server, session, envelope, hostname):
         session.host_name = hostname
-        yield from server.push('250-AUTH PLAIN')
+        await server.push('250-AUTH PLAIN')
         return '250 HELP'
 
-    @asyncio.coroutine
-    def handle_RSET(self, server, session, envelope):
+    async def handle_RSET(self, server, session, envelope):
         self.connection_count = 0
         return '250 OK'
 
@@ -83,8 +80,7 @@ class ConnectionCountingSMTP(SMTP):
         # same though, so it's fine to stash this value away there.
         self.event_handler.connection_count += 1
 
-    @asyncio.coroutine
-    def smtp_AUTH(self, arg):
+    async def smtp_AUTH(self, arg):
         """Record that the AUTH occurred."""
         args = arg.split()
         if args[0].lower() == 'plain':
@@ -94,25 +90,24 @@ class ConnectionCountingSMTP(SMTP):
                 # which must be equal to the base 64 equivalent of the
                 # expected login string "testuser:testpass".
                 if response == 'AHRlc3R1c2VyAHRlc3RwYXNz':
-                    yield from self.push('235 Ok')
+                    await self.push('235 Ok')
                     self._oob_queue.put(response)
                 else:
-                    yield from self.push('571 Bad authentication')
+                    await self.push('571 Bad authentication')
             else:
                 assert len(args) == 1, args
                 # Send a challenge and set us up to wait for the response.
-                yield from self.push('334 ')
+                await self.push('334 ')
                 self._waiting_for_auth_response = True
         else:
-            yield from self.push('571 Bad authentication')
+            await self.push('571 Bad authentication')
 
-    @asyncio.coroutine
-    def smtp_STAT(self, arg):
+    async def smtp_STAT(self, arg):
         """Cause the server to send statistics to its controller."""
         # Do not count the connection caused by the STAT connect.
         self.event_handler.connection_count -= 1
         self._oob_queue.put(self.event_handler.connection_count)
-        yield from self.push('250 Ok')
+        await self.push('250 Ok')
 
     def _next_error(self, command):
         """Return the next error for the SMTP command, if there is one.
@@ -139,40 +134,40 @@ class ConnectionCountingSMTP(SMTP):
             return code
         return None
 
-    @asyncio.coroutine
-    def smtp_RCPT(self, arg):
+    async def smtp_RCPT(self, arg):
         """For testing, sometimes cause a non-25x response."""
         code = self._next_error('rcpt')
         if code is None:
             # Everything's cool.
-            yield from super().smtp_RCPT(arg)
+            await super().smtp_RCPT(arg)
         else:
             # The test suite wants this to fail.  The message corresponds to
             # the exception we expect smtplib.SMTP to raise.
-            yield from self.push('%d Error: SMTPRecipientsRefused' % code)
+            await self.push('%d Error: SMTPRecipientsRefused' % code)
 
-    @asyncio.coroutine
-    def smtp_MAIL(self, arg):
+    async def smtp_MAIL(self, arg):
         """For testing, sometimes cause a non-25x response."""
         code = self._next_error('mail')
         if code is None:
             # Everything's cool.
-            yield from super().smtp_MAIL(arg)
+            await super().smtp_MAIL(arg)
         else:
             # The test suite wants this to fail.  The message corresponds to
             # the exception we expect smtplib.SMTP to raise.
-            yield from self.push('%d Error: SMTPResponseException' % code)
+            await self.push('%d Error: SMTPResponseException' % code)
 
 
 class ConnectionCountingController(Controller):
     """Count the number of SMTP connections opened."""
 
-    def __init__(self, host, port):
+    def __init__(self, host, port, ssl_context=None):
         self._msg_queue = Queue()
         self._oob_queue = Queue()
         self.err_queue = Queue()
+
         handler = ConnectionCountingHandler(self._msg_queue)
-        super().__init__(handler, hostname=host, port=port)
+        super().__init__(handler, hostname=host, port=port,
+                         ssl_context=ssl_context)
 
     def factory(self):
         return ConnectionCountingSMTP(
@@ -185,8 +180,7 @@ class ConnectionCountingController(Controller):
         self.reset()
 
     def _connect(self):
-        client = smtplib.SMTP()
-        client.connect(self.hostname, self.port)
+        client = smtplib.SMTP(self.hostname, self.port)
         return client
 
     def get_connection_count(self):
@@ -224,3 +218,47 @@ class ConnectionCountingController(Controller):
     def reset(self):
         client = self._connect()
         client.docmd('RSET')
+
+
+class ConnectionCountingSSLController(ConnectionCountingController):
+    """Count the number of SMTPS connections opened."""
+
+    def __init__(self, host, port, client_context=None, server_context=None):
+        super().__init__(host, port, ssl_context=server_context)
+        self._client_context = client_context
+
+    def _connect(self):
+        client = smtplib.SMTP_SSL(self.hostname, self.port,
+                                  context=self._client_context)
+        return client
+
+
+class ConnectionCountingSTARTTLSController(ConnectionCountingController):
+    """Count the number of SMTP connections with STARTTLS opened."""
+
+    def __init__(self, host, port, client_context=None, server_context=None):
+        super().__init__(host, port)
+        self._client_context = client_context
+        self._server_context = server_context
+
+    def factory(self):
+        return ConnectionCountingSMTP(
+            self.handler, self._oob_queue,
+            self.err_queue, tls_context=self._server_context,
+            require_starttls=True)
+
+    def _connect(self):
+        client = smtplib.SMTP(self.hostname, self.port)
+        client.starttls(context=self._client_context)
+        return client
+
+    def get_connection_count(self):
+        count = super().get_connection_count()
+        # This is a hack, since when using STARTTLS for every connection the
+        # SMTP.connection_made(transport) method is called twice.
+        # The -1 is there for the last pair of connections made when checking
+        # the connection count, since one of them is already subtracted in
+        # ConnectionCountingSMTP.handle_STAT.
+        # The //2 is there for the rest of the connections which are counted
+        # twice.
+        return (count - 1) // 2
